import errno
import json
import logging
import os

from dateutil import parser
from .util import get_json_log_data_interval

from zope.interface import implementer
from slapos.grid.promise import interface
from slapos.grid.promise.generic import GenericPromise

@implementer(interface.IPromise)
class RunPromise(GenericPromise):

  def __init__(self, config):

    self.__name = config.get('name', None)
    self.__log_folder = config.get('log-folder', None)

    super(RunPromise, self).__init__(config)
    self.setPeriodicity(minute=1)

    self.__title = os.path.splitext(self.__name)[0]
    self.__log_file = os.path.join(self.__log_folder, '%s.json.log' % self.__title)

    self.json_logger = logging.getLogger('json_logger')
    self.json_logger.setLevel(logging.INFO)
    handler = logging.FileHandler(self.__log_file)
    formatter = logging.Formatter('{"time": "%(asctime)s", "log_level": "%(levelname)s", "message": "%(message)s", "data": %(data)s}')
    handler.setFormatter(formatter)
    self.json_logger.addHandler(handler)

  def sense(self):

    testing = self.getConfig('testing') == "True"
    if testing:
        self.logger.info("skipping promise")
        return
    amarisoft_stats_log = self.getConfig('amarisoft-stats-log')
    stats_period = int(self.getConfig('stats-period'))
    min_rxtx_delay_threshold = float(self.getConfig('min-rxtx-delay', 0))

    data_list = get_json_log_data_interval(amarisoft_stats_log, stats_period * 2)

    min_rxtx_delay_it = map(lambda x: float(x['rf']['rxtx_delay_min']), data_list)
    if not min_rxtx_delay_it:
        self.logger.error("No TX/RX diff data available")
    else:
      min_rxtx_delay = min(min_rxtx_delay_it)
      if min_rxtx_delay < min_rxtx_delay_threshold:
        self.logger.error("The minimum available time %s (ms) for radio front end processing is lower than the threshold %s (ms)." % (min_rxtx_delay, min_rxtx_delay_threshold))
      else:
        self.logger.info("The minimum %s (ms) available time for radio front end processing is OK" % (min_rxtx_delay,))

    self.json_logger.info("Minimum available time for radio front end processing (ms)",
      extra={'data': {'min_rxtx_delay': min_rxtx_delay}})

  def test(self):
    """
      Called after sense() if the instance is still converging.
      Returns success or failure based on sense results.

      In this case, fail if the previous sensor result is negative.
    """
    return self._test(result_count=1, failure_amount=1)


  def anomaly(self):
    """
      Called after sense() if the instance has finished converging.
      Returns success or failure based on sense results.
      Failure signals the instance has diverged.

      In this case, fail if two out of the last three results are negative.
    """
    return self._anomaly(result_count=1, failure_amount=1)
